// -*- C++ -*-
// AID-GENERATED
// =========================================================================
// This class was generated by AID - Abstract Interface Definition          
// DO NOT MODIFY, but use the org.freehep.aid.Aid utility to regenerate it. 
// =========================================================================
#ifndef IO_LCREADER_H
#define IO_LCREADER_H 1

#include <string>
#include <vector>

#include "EVENT/LCEvent.h"
#include "EVENT/LCRunHeader.h"
#include "Exceptions.h"

namespace IO {

class LCEventListener;
class LCRunListener;

/**Interface for reading data from LCIO.
 * Use LCFactory to instantiate a corresponding LCWriter object
 * for the output format at hand (SIO only, so far).
 *
 * @see LCFactory
 * 
 * @author gaede
 * @version Mar 4, 2003
 */
class LCReader {

public: 
    /// Destructor.
    virtual ~LCReader() { /* nop */; }

    /** Opens a file for reading (read-only).
     *
     * @throws IOException
     */
    virtual void open(const std::string & filename) throw (IOException, std::exception )  = 0;

    /** Opens a list of files for reading (read-only). All subsequent
     * read operations will operate on the list, i.e. if an EOF is encountered
     * the next file in the list will be opened and read transparently to the
     * user.
     *
     * @throws IOException
     */
    virtual void open(const std::vector<std::string>  & filenames) throw (IOException, std::exception )  = 0;

    /** Reads the next run header from the file. Returns NULL if
     * 'EOF' read. 
     *
     * @throws IOException
     */
    virtual EVENT::LCRunHeader * readNextRunHeader() throw (IOException, std::exception )  = 0;

    /** Same as readNextRunHeader() but allows to set the access mode 
     *  LCIO::READ_ONLY (default) or LCIO::Update. 
     *
     * @throws IOException
     */
    virtual EVENT::LCRunHeader * readNextRunHeader(int accessMode) throw (IOException, std::exception )  = 0;

    /** Reads the next event from the file. Returns NULL if
     * 'EOF' read. 
     *
     * @throws IOException
     */
    virtual EVENT::LCEvent * readNextEvent() throw (IOException, std::exception )  = 0;

    /** Same as readNextEvent() but allows to set the access mode 
     *  LCIO::READ_ONLY (default) or LCIO::Update. 
     *
     * @throws IOException
     */
    virtual EVENT::LCEvent * readNextEvent(int accessMode) throw (IOException, std::exception )  = 0;

    /** Skips the next n events from the current position.
     */
    virtual void skipNEvents(int n) = 0;

    /** Reads the specified event from file. 
     *  To be used with care: events have to be read in sequential 
     *  order (as LCIO has no direct access yet).  Returns NULL if
     * 'EOF' read, i.e. the specified event hasn't been found in the file.
     *
     * @throws IOException
     */
    virtual EVENT::LCEvent * readEvent(int runNumber, int evtNumber) throw (IOException, std::exception )  = 0;

    /** Closes the output file/stream etc.
     *
     * @throws IOException
     */
    virtual void close() throw (IOException, std::exception )  = 0;

    /** Registers a listener for reading LCEvents from a stream.
     */ 
    virtual void registerLCEventListener(LCEventListener * ls) = 0;

    /** Remove a listener for reading LCEvents from a stream.
     */ 
    virtual void removeLCEventListener(LCEventListener * ls) = 0;

    /** Registers a listener for reading LCEventsLCRunHeaders from a stream.
     */ 
    virtual void registerLCRunListener(LCRunListener * ls) = 0;

    /** Remove a listener for reading LCRunHeaders from a stream.
     */ 
    virtual void removeLCRunListener(LCRunListener * ls) = 0;

    /** Reads the input stream and notifies registered 
     * listeners according to the object type 
     * found in the stream. 
     *
     * @throws IOException
     */
    virtual void readStream() throw (IOException, std::exception )  = 0;

    /** Reads maxRecord from the input stream and notifies registered 
     * listeners according to the object type found in the stream.
     * An exception is thrown if less than maxRecord where read from the file.
     *
     * @throws IOException
     */
    virtual void readStream(int maxRecord) throw (IOException, std::exception )  = 0;
}; // class
} // namespace IO
#endif /* ifndef IO_LCREADER_H */
